//----------------
// Enemy class
//----------------
var hurtColorFilter = new ColorFilter(13, 13, 0, 13);
var normalColorFilter = new ColorFilter(1, 1, 1, 1);

var enemySS = new SpriteSheet({
    images: ["./img/enemy0.png", "./img/enemy1.png", "./img/enemy2.png", "./img/enemy3.png", "./img/enemy4.png"],
    frames: {width:200, height:200},
    animations: {
        "small":{
        	frames:[0,1,2,3],
        	speed : 0.2,
        	next:true,
    	},
        "medium":{
        	frames:[4],
        	next:false,
    	},
        "big":{
        	frames:[5],
        	next:false,
    	},
        "donut":{
        	frames:[6],
        	next:false,
    	},
        "ramen":{
        	frames:[7],
        	next:false,
    	}
    }
});

(function() {
	
	//--------------
	// constructor
	//--------------
	var Enemy = function() {
		this.initialize(enemySS, "");
		this.exist = false;
		this.regX = 100;
		this.regY = 100;
		this.y = -1000;
		
		// tick
		this.on("tick", this.tick);
	};
	
	// inherit
	var p = Enemy.prototype = new Sprite();
	
	// property
	p.cnt = 0;
	p.exist = false;
	p.size = 0;
	p.velocityX = 0;
	p.velocityY = 0;
	p.type = 0;
	p.hp = 0;
	p.hurt = 0;
	p.score = 0;
	
	//--------
	// tick
	//--------
	p.tick = function(){
		
		if(this.exist){
			
			this.cnt ++;
			
			// move
			this.x += this.velocityX;
			this.y += this.velocityY;
			
			if(this.type == "donut"){
				this.x += Math.cos(this.cnt / 25) * 6;
			}else if(this.type == "ramen"){
				this.x += Math.cos(this.cnt / 50) * 2;
			}
			
			// damage
			if(this.y > 900){
				for(var i=0; i<this.hurt; i++){
					damage();
				}
				this.dead();
			}
		}
	}
	
	//-----------
	// put
	//-----------
	p.put = function(inX, inY, inVelX, inVelY, inType){
		this.gotoAndPlay(inType);
		this.velocityX = inVelX;
		this.velocityY = inVelY;
		this.x = inX;
		this.y = inY;
		this.exist = true;
		this.cache(0, 0, this.getBounds().width, this.getBounds().height);
		
		// type
		this.type = inType;
		switch(this.type){
			case "small":
				this.hp = 1;
				this.size = 50;
				this.hurt = 1;
				this.uncache();
				break;
			case "medium":
				this.hp = 10;
				this.hurt = 2;
				this.size = 80;
				break;
			case "big":
				this.hp = 50;
				this.hurt = 5;
				this.size = 100;
				break;
			case "donut":
				this.hp = 5;
				this.hurt = 2;
				this.size = 80;
				break;
			case "ramen":
				this.hp = 200;
				this.hurt = 10;
				this.size = 100;
				break;
		}
		this.score = this.hp * 10;
	}
	
	//------
	// hit
	//------
	p.hit = function(inDamage){
		
		// color
		this.filters = [hurtColorFilter];
		this.cache(0, 0, this.getBounds().width, this.getBounds().height);
		var _this = this;
		setTimeout(function(){
			_this.filters = [normalColorFilter];
			_this.cache(0, 0, _this.getBounds().width, _this.getBounds().height);
		}, 20);
		
		// reaction
		if(this.y > 0){
			this.y -= this.velocityY + 10 * (100 / this.size);
		}
		
		// reduce hp
		this.hp -= inDamage;
		
		// dead
		if(this.hp <= 0){
			
			var explosionNum = 0;
			switch(this.type){
				case "medium":
					explosionNum = 4;
					soundExplosion1.play();
					shake(10, 4);
					break;
				case "big":
					explosionNum = 8;
					soundExplosion2.play();
					shake(15, 8);
					break;
				case "ramen":
					explosionNum = 16;
					soundExplosion2.play();
					shake(15, 12);
					break;
				default:
					explosionNum = 2;
					soundExplosion1.play();
					break;
			}
			
			for(var i=0; i<explosionNum; i++){
				setTimeout(function(inX, inY, inSize){
					putEmitter(
						inX + getIntRand(-inSize/2, inSize/2),
						inY + getIntRand(-inSize/2, inSize/2),
						getIntRand(-5, 6),
						getIntRand(-5, 6),
						"explosion",
						true
					);
				}, i * 40, this.x, this.y, this.size);
			}
			
			gameScore += this.score;
			this.dead();
		}
	}
	
	//---------
	// dead
	//---------
	p.dead = function(){
		this.exist = false;
		this.y = -1000;
	}
	
	// regist
	createjs.Enemy = Enemy;
}());
