
// load files
var loadManifest = [
	"./img/title.png",
	"./img/ok.png",
	"./img/bg.png",
	"./img/beam.png",
	"./img/tooth.png",
	"./img/toothpaste.png",
	"./img/enemy1.png",
	"./img/enemy2.png",
	"./sound/shot.mp3",
	"./sound/explosion1.mp3",
	"./sound/explosion2.mp3",
	"./sound/hit.mp3",
	"./sound/damage.mp3",
	"./sound/launch.mp3",
	"./sound/overheat.mp3",
	"./sound/bgm.mp3",
];

// sound
var soundShot;
var soundExplosion1;
var soundExplosion2;
var soundHit;
var soundDamage;
var soundLaunch;
var soundOverheat;
var soundBGM;

// ss
var toothpasteSS = new SpriteSheet({
    images: ["./img/toothpaste.png"],
    frames: {width:100, height:50},
});

// shift
shiftKeyDown = false;
document.onkeydown = function(event) {
	if(event.shiftKey) {
		shiftKeyDown = true;
	}
} 
document.onkeyup = function(event) {
	shiftKeyDown = false;
} 

//------------------------------
// game init(global property)
//------------------------------
function gameInit(){
	alive = true;
	gameScore = 0;
	health = 16;
	gameCnt = 0;
	stageLevel = 0;
	mouseCnt = -1;
	mousex = 0;
	mousey = 0;
	beamIdx = 0;
	enemyIdx = 0;
	emitterIdx = 0;
	heat = 0;
	heatTime = 0;
	overheat = false;
	healthyTeeth = shuffle([0, 1, 2, 3, 4, 5, 6, 7]);
	okButton.visible = false;
	upperText.text = "";
	centerText.text = "";
	
	for(var i=0; i<beamContainer.children.length; i++){
		beamContainer.getChildAt(i).dead();
	}
	for(var i=0; i<enemyContainer.children.length; i++){
		enemyContainer.getChildAt(i).dead();
	}
	for(var i=0; i<emitterContainer.children.length; i++){
		emitterContainer.getChildAt(i).dead();
	}
	
	// theeth
	teethContainer.removeAllChildren();
	for(var i=0; i<health/2; i++){
		var tooth = new Tooth(i);
		tooth.x = 25 + i*50 + (600 / 2 - (50 * (health/2)) / 2);
		tooth.y = 800 - 25;
		teethContainer.addChild(tooth);
	}
	
	// bgm
	soundBGM.play({loop:-1});
}

//-------------
// load start
//-------------
window.onload = function(){
	
	// stage
	stage = new Stage(document.getElementById("stage"));
	
	// loading text
	loadingText = new Text("Loading...", "40px Arial", "#FFF");
	loadingText.x = 20;
	loadingText.y = 20;
	stage.addChild(loadingText);
	
	// load start
	var loader = new LoadQueue();
	loader.on("complete", loadComplete);
	loader.loadManifest(loadManifest);
	
	stage.update();
}

//----------------
// load complete
//----------------
function loadComplete(){
	
	// sound
	soundShot = Sound.createInstance("./sound/shot.mp3");
	soundExplosion1 = Sound.createInstance("./sound/explosion1.mp3");
	soundExplosion2 = Sound.createInstance("./sound/explosion2.mp3");
	soundHit = Sound.createInstance("./sound/hit.mp3");
	soundDamage = Sound.createInstance("./sound/damage.mp3");
	soundLaunch = Sound.createInstance("./sound/launch.mp3");
	soundOverheat = Sound.createInstance("./sound/overheat.mp3");
	soundBGM = Sound.createInstance("./sound/bgm.mp3");
	
	// volume init
	var masterVolume = window.localStorage.getItem("teeth_war_volume");
	if(masterVolume){
		document.getElementById("range_volume").value = masterVolume;
	}else{
		document.getElementById("range_volume").value = 30;
	}
	changeVolume();
	
	// scene container
	titleContainer = new Container();
	gameContainer = new Container();
	
	//--------
	// title
	//--------
	titleBmp1 = new Bitmap("./img/title.png");
	titleBmp1.regX = 350;
	titleBmp1.regY = 450;
	titleBmp1.x = 300;
	titleBmp1.y = 400;
	titleBmp1.on("click", function(evt){
		switchScene("game");
		
		// score attack
		if(shiftKeyDown){
			stageLevel = 10;
			centerText.text = "SCORE ATTACK";
		}
	});
	titleContainer.addChild(titleBmp1);
	
	//--------
	// game
	//--------
	// bg
	bgBmp1 = new Bitmap("./img/bg.png");
	bgBmp1.regX = 350;
	bgBmp1.regY = 450;
	bgBmp1.x = 300;
	bgBmp1.y = 400;
	gameContainer.addChild(bgBmp1);
	
	// tooth paste
	toothpasteLeft = new Sprite(toothpasteSS);
	toothpasteLeft.regX = 20;
	toothpasteLeft.regY = 25;
	toothpasteLeft.x = 0;
	toothpasteLeft.y = 800;
	gameContainer.addChild(toothpasteLeft);
	
	toothpasteRight = new Sprite(toothpasteSS);
	toothpasteRight.regX = 20;
	toothpasteRight.regY = 25;
	toothpasteRight.x = 600;
	toothpasteRight.y = 800;
	gameContainer.addChild(toothpasteRight);
	
	// enemy container
	enemyContainer = new Container();
	for(var i=0; i<20; i++){
		var enemy = new Enemy();
		enemyContainer.addChild(enemy);
	}
	gameContainer.addChild(enemyContainer);
	
	// beam container
	beamContainer = new Container();
	for(var i=0; i<20; i++){
		var beam = new Beam();
		beamContainer.addChild(beam);
	}
	gameContainer.addChild(beamContainer);
	
	// teeth container
	teethContainer = new Container();
	gameContainer.addChild(teethContainer);
	
	// emitter container
	emitterContainer = new Container();
	for(var i=0; i<20; i++){
		var emitter = new Emitter();
		emitterContainer.addChild(emitter);
	}
	gameContainer.addChild(emitterContainer);
	
	// upper text
	upperText = new Text("", "30px Arial", "#FFF");
	upperText.x = 300;
	upperText.y = 350;
	upperText.textAlign = "center"; 
	gameContainer.addChild(upperText);
	
	// center text
	centerText = new Text("", "60px Arial", "#FFF");
	centerText.x = 300;
	centerText.y = 200;
	centerText.textAlign = "center"; 
	gameContainer.addChild(centerText);
	
	// meter
	meterBar = new Shape();
	meterBar.graphics.beginFill("#0CF").drawRect(0, 0, 400, 20);
	meterBar.x = 100;
	meterBar.y = 20;
	gameContainer.addChild(meterBar);
	
	meterFrame = new Shape();
	meterFrame.graphics.setStrokeStyle(2);
	meterFrame.graphics.beginStroke("#000").drawRect(0, 0, 400, 20);
	meterFrame.x = 100;
	meterFrame.y = 20;
	gameContainer.addChild(meterFrame);
	
	// fader
	fader = new Shape();
	gameContainer.addChild(fader);
	
	// ok button
	okButton = new Bitmap("./img/ok.png");
	okButton.regX = 100;
	okButton.regY = 40;
	okButton.x = 300;
	okButton.y = 600;
	okButton.on("click", function(){
		switchScene("title");
	});
	gameContainer.addChild(okButton);
	
	//----------
	// mouse
	//----------
	stage.on("stagemousedown", function(evt) {
		mouseCnt = 0;
	});
	stage.on("stagemouseup", function(evt) {
		mouseCnt = -1;
	});
	stage.on("stagemousemove", function(evt) {
		mousex = evt.stageX;
		mousey = evt.stageY;
	});
	
	// ticker
	Ticker.setFPS(60);
	
	// switch
	setTimeout(function(){
		stage.removeChild(loadingText);
		switchScene("title");
	}, 500);
}

//--------------
// switch scene
//--------------
function switchScene(inScene){
	switch(inScene){
		case "title":
			stage.addChild(titleContainer);
			stage.removeChild(gameContainer);
			Ticker.removeEventListener("tick", gameTick);
			soundBGM.stop();
			break;
			
		case "game":
			gameInit();
			stage.removeChild(titleContainer);
			stage.addChild(gameContainer);
			Ticker.addEventListener("tick", gameTick);
			break;
	}
	stage.update();
}

//------------
// game tick
//------------
function gameTick(){
	
	// game count
	gameCnt ++;
	
	if(alive){
		
		// beam fire
		if(mouseCnt >= 0 && mouseCnt ++ % 3 == 0 && overheat == false){
			soundShot.play();
			var randSize = 50;
			var beam1 = beamContainer.getChildAt(beamIdx);
			var beam2 = beamContainer.getChildAt(beamIdx + 1);
			beam1.fire(
				0 + getIntRand(-randSize, randSize),
				800 + getIntRand(-randSize, randSize),
				mousex,
				mousey - 40);
			beam2.fire(
				600 + getIntRand(-randSize, randSize),
				800 + getIntRand(-randSize, randSize),
				mousex,
				mousey - 40);
			beamIdx += 2;
			if(beamIdx >= beamContainer.children.length){
				beamIdx = 0;
			}
			
			toothpasteLeft.gotoAndStop(1);
			toothpasteLeft.x = getIntRand(-20,20);
			toothpasteLeft.y = getIntRand(-20,20) + 800;
			toothpasteRight.gotoAndStop(1);
			toothpasteRight.x = getIntRand(-20,20) + 600;
			toothpasteRight.y = getIntRand(-20,20) + 800;
			setTimeout(function(){
				toothpasteLeft.gotoAndStop(0);
				toothpasteLeft.x = 0;
				toothpasteLeft.y = 800;
				toothpasteRight.gotoAndStop(0);
				toothpasteRight.x = 600;
				toothpasteRight.y = 800;
			}, 40);
			heat += 2;
			heatTime = 5;
			if(heat >= 100){
				heat = 100;
				overheat = true;
				centerText.text = "OVERHEAT!!";
				soundOverheat.play();
			}
		}
		
		// heat bar
		if(heat > 0){
			
			if(heatTime -- < 0){
				heat --;
			}
			if(heat <= 0){
				heat = 0;
				overheat = false;
				centerText.text = "";
			}
		}
		meterBar.scaleX = heat / 100;
		
		// tooth paste
		var lr = Math.atan2(mousey - toothpasteLeft.y, mousex - toothpasteLeft.x);
		toothpasteLeft.rotation = lr * 180 / Math.PI;
		var rr = Math.atan2(mousey - toothpasteRight.y, mousex - toothpasteRight.x);
		toothpasteRight.rotation = rr * 180 / Math.PI;
	}
	
	// enemy spawn
	if(gameCnt % 20 == 0){
		var enemy;
		for(var ei=0; ei<enemyContainer.children.length; ei ++){
			enemy = enemyContainer.getChildAt(enemyIdx);
			enemyIdx ++;
			if(enemyIdx >= enemyContainer.children.length){
				enemyIdx = 0;
			}
			if(enemy.exist == false){
				
				// stage level
				switch(stageLevel){
					case 0:
						if(Math.random() < 0.8){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(3, 6), "small");
						}else{
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(2, 4), "medium");
						}
						break;
						
					case 1:
						if(Math.random() < 0.7){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(3, 8), "small");
						}else if(Math.random() < 0.7){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(3, 6), "medium");
						}else{
							enemy.put(300, -100, 0, getIntRand(2, 4), "donut");
						}
						if(gameCnt%400 == 0){
							enemy.put(getIntRand(100, 500), -100, 0, 2, "big");
						}
						break;
						
					case 2:
						if(Math.random() < 0.7){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(4, 10), "small");
						}else if(Math.random() < 0.5){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(4, 8), "medium");
						}else{
							enemy.put(300, -100, 0, getIntRand(3, 6), "donut");
						}
						if(gameCnt%400 == 0){
							enemy.put(getIntRand(100, 500), -100, 0, 2, "big");
						}
						if(gameCnt%1200 == 0){
							enemy.put(300, -100, 0, 1, "ramen");
						}
						break;
						
					case 10:
						if(Math.random() < 0.8){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(4, 10), "small");
						}else if(Math.random() < 0.6){
							enemy.put(getIntRand(20, 580), -100, 0, getIntRand(4, 8), "medium");
						}else if(Math.random() < 0.9){
							enemy.put(300, -100, 0, getIntRand(3, 8), "donut");
						}else if(Math.random() < 0.7){
							enemy.put(getIntRand(100, 500), -100, 0, 2, "big");
						}else{
							enemy.put(300, -100, 0, 1, "ramen");
						}
						if(gameCnt%500 == 0){
							enemy.put(getIntRand(100, 500), -100, 0, 2, "big");
						}
						if(gameCnt%1200 == 0){
							enemy.put(300, -100, 0, 1, "ramen");
						}
						break;
				}
				
				break;
			}
		}
	}
	
	// stage level up
	if(stageLevel < 10){
		if(gameCnt == 800){
			stageLevel ++;
		}else if(gameCnt == 2400){
			stageLevel ++;
		}else if(gameCnt == 4000){
			stageLevel ++;
		}else if(gameCnt > 4000){
			
			// victory if no enemies
			if(gameCnt % 50 == 0){
				
				var existEnemy = false;
				for(var i=0; i<enemyContainer.children.length; i++){
					if(enemyContainer.getChildAt(i).exist){
						existEnemy = true;
						break;
					}
				}
				if(existEnemy == false){
					centerText.text = "VICTORY!";
					upperText.text = "Score Attack next!\nStart with a Shift Key.";
					alive = false;
					okButton.visible = true;
				}
			}
		}
	}
	
	// enemy collision
	for(var ei=0; ei<enemyContainer.children.length; ei++){
		var enemy = enemyContainer.getChildAt(ei);
		if(enemy.exist == false){
			continue;
		}
		
		// beam collision
		for(var bi=0; bi<beamContainer.children.length; bi++){
			var beam = beamContainer.getChildAt(bi);
			if(beam.exist == false){
				continue;
			}
			var distance = Math.sqrt(Math.pow(beam.x - enemy.x, 2) + Math.pow(beam.y - enemy.y, 2));
			if(distance <= beam.size + enemy.size){
				
				// emitter
				var radian = Math.atan2(beam.y - enemy.y, beam.x - enemy.x);
				putEmitter(
					enemy.x + Math.cos(radian) * enemy.size,
					enemy.y + Math.sin(radian) * enemy.size,
					0,
					0,
					"hit",
					true
				);
				
				soundHit.play();
				enemy.hit(1);
				beam.dead();
			}
		}
		
		// teeth collision
		for(var ti=0; ti<teethContainer.children.length; ti++){
			var thooth = teethContainer.getChildAt(ti);
			if(thooth.exist == false){
				continue;
			}
			var distance = Math.sqrt(Math.pow(thooth.x - enemy.x, 2) + Math.pow(thooth.y - enemy.y, 2));
			if(distance <= thooth.size + enemy.size){
				enemy.hit(10);
			}
		}
	}
	
	// draw
	stage.update();
}

//----------
// damage
//----------
var damagedTooth = null;
function damage(){
	if(health <= 0){
		return;
	}
	
	health --;
	fade("#00F", 0.5, 0.03);
	shake(20, 5);
	soundDamage.play();
	
	// pick random tooth
	if(damagedTooth == null){
		damagedTooth = teethContainer.getChildAt(healthyTeeth[0]);
		damagedTooth.gotoAndStop(1);
		
	// damaged tooth
	}else{
		damagedTooth.gotoAndStop(2);
		damagedTooth = null;
		healthyTeeth.shift();
	}
	
	// gameover
	if(health <= 0){
		alive = false;
		okButton.visible = true;
		soundBGM.stop();
		soundOverheat.play();
		
		if(stageLevel < 10){
			centerText.text = "GAMEOVER";
		}else{
			centerText.text = "SCORE: " + gameScore;
		}
	}
}

//------------
// pull tooth
//------------
function pullTooth(inNum){
	health -= 2;
	for(var i=0; i<healthyTeeth.length; i++){
		if(healthyTeeth[i] == inNum){
			healthyTeeth.splice(i, 1);
			return;
		}
	}
}

//----------
// shake
//----------
function shake(inSize, inDuration){
	for(var i=0; i<inDuration; i++){
		setTimeout(function(){
			gameContainer.x = getIntRand(-inSize, inSize);
			gameContainer.y = getIntRand(-inSize, inSize);
		}, 40 * i);
	}
	setTimeout(function(){
		gameContainer.x = 0;
		gameContainer.y = 0;
	}, 40 * inDuration);
}

//----------
// emitter
//----------
function putEmitter(inX, inY, inVelX, inVelY, inType, inRotation){
	
	var emitter = emitterContainer.getChildAt(emitterIdx);
	emitter.put(inX, inY, inVelX, inVelY, inType, inRotation);
	emitterIdx ++;
	if(emitterIdx >= emitterContainer.children.length){
		emitterIdx = 0;
	}
}

//-----------
// fade
//-----------
var fadeListener;
function fadeTick(){
	fader.alpha -= fader.duration;
	if(fader.alpha <= 0){
		fader.off("tick", fadeListener);
		fader.graphics.clear();
	}
}

function fade(inColor, inStartAlpha, inDuration){
	fader.graphics.clear();
	fader.graphics.beginFill(inColor).drawRect(-100, -100, 800, 1000);
	fader.alpha = inStartAlpha;
	fader.duration = inDuration;
	fader.off("tick", fadeListener);
	fadeListener = fader.on("tick", fadeTick);
}

//-----------
// random
//-----------
function getIntRand(inStart, inEnd){
	return Math.floor(Math.random() * (inEnd - inStart)) + inStart;
}

//-----------------
// volume
//-----------------
function changeVolume(){
	var inVolume = document.getElementById("range_volume").value;
	window.localStorage.setItem("teeth_war_volume", inVolume);
	Sound.setVolume(inVolume / 100);
}

//-----------------
// shuffle array
//-----------------
function shuffle(array) {
  var n = array.length, t, i;
  while (n) {
    i = Math.floor(Math.random() * n--);
    t = array[n];
    array[n] = array[i];
    array[i] = t;
  }
  return array;
}

